"""Create a Volume from a numpy array"""
import numpy as np

data_matrix = np.zeros([75, 75, 75], dtype=np.uint8)
# all voxels have value zero except:
data_matrix[0:35,   0:35,  0:35] = 1
data_matrix[35:55, 35:55, 35:55] = 2
data_matrix[55:74, 55:74, 55:74] = 3

from vedo import Volume, show

vol = Volume(data_matrix, c=['white','b','g','r'], mapper='gpu')
vol.addScalarBar3D()

# optionally mask some parts of the volume (needs mapper='gpu'):
# data_mask = np.zeros_like(data_matrix)
# data_mask[10:65, 10:45, 20:75] = 1
# vol.mask(data_mask)

show(vol, __doc__, axes=1).close()



"""Segment a TetMesh with a custom scalar.
Press q to make it explode"""
from vedo import Mesh, TetMesh, Plotter, Text2D, dataurl
import tetgen
import pymeshfix

n = 20000
f1 = 0.005  # control the tetras resolution
f2 = 0.15   # control the nr of seeds

# repair and tetralize the closed surface
amesh = Mesh(dataurl+'bunny.obj')
meshfix = pymeshfix.MeshFix(amesh.points(), amesh.faces())
meshfix.repair() # will make it manifold
repaired = Mesh(meshfix.mesh)
tet = tetgen.TetGen(repaired.points(), repaired.faces())
tet.tetrahedralize(order=1, mindihedral=50, minratio=1.5)
tmesh = TetMesh(tet.grid)

surf = tmesh.tomesh(fill=False)
txt = Text2D(__doc__, font="Brachium")

# pick points on the surface and use subsample to make them uniform
seeds = surf.clone().subsample(f2).ps(10).c('black')

# assign to each tetrahedron the id of the closest seed point
cids = []
for p in tmesh.cellCenters():
	cid = seeds.closestPoint(p, returnPointId=True)
	cids.append(cid)
tmesh.celldata["fragment"] = cids
#tmesh.celldata.select("fragment")# bug, has no effect, needs name=...

pieces = []
for i in range(seeds.NPoints()):
	tc = tmesh.clone().threshold(name="fragment", above=i-0.1, below=i+0.1)
	mc = tc.tomesh(fill=False).color(i)
	pieces.append(mc)

############### animate
plt = Plotter(size=(1200,800), axes=1)
plt.show(txt, pieces)
for i in range(10):
    for pc in pieces:
        cm = pc.centerOfMass()
        pc.shift(cm/25)
    txt.text(f"{__doc__}\n\nNr. of pieces = {seeds.N()}")
    plt.render()
plt.interactive().close()






#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# https://www.redblobgames.com/articles/visibility/
# https://en.wikipedia.org/wiki/Art_gallery_problem
"Hover mouse to illuminate the visible area of the room"
from vedo import *
import numpy as np

rw = Rectangle((0,0), (1,1)).texture(dataurl+"textures/paper1.jpg")
tx = Text3D("эd", font="Comae").pickable(False)
tx.scale(0.475).pos(0.1, 0.2, 0.0002).c('green4')
d = rw.diagonalSize()

objs = [rw, tx]
mobjs = merge(objs).c('grey4').flat()
allpts = mobjs.points()
walls = mobjs.extrude(0.03, cap=False).z(-0.01).pickable(False).flat()

def func(evt):
	p = evt.picked3d
	if p is None:
		return

	pts1 = []
	for q1 in allpts:
		v = versor(q1-p)
		e = cross(v, [0,0,1]) * 0.01  # a small epsilon shift
		for k in (v, v+e, v-e):
			ipts = walls.intersectWithLine(p, p+k*d, tol=1e-04)
			n = len(ipts)
			if n==1 or n>1 and mag(q1-ipts[0]) < 1e-04:
				pts1.append(ipts[0])

	pts2 = []
	for pt in pts1:
		angle = np.arctan2(pt[1]-p[1], pt[0]-p[0])
		pts2.append([pt, angle])
	pts = utils.sortByColumn(pts2,1)[:,0].tolist() # sort by angle

	line = Line(pts, closed=True).z(0.01).lw(4).c('grey3')
	surf = line.clone().triangulate().lw(0).c('yellow4').alpha(0.3)
	surf.pickable(False)
	area = Assembly(surf, line)
	area.name = "Area"
	plt.remove("Area").add(area)

plt = Plotter(bg2="light blue")
plt.addCallback("mouse hover", func)
plt.show(objs, walls, __doc__, zoom=1.1, elevation=-20).close()


